@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005-2008, 2010-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005-2008, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,10 +35,7 @@ comp (const void *item, const void *data)
 void
 auth_add (char *name, imap4d_auth_handler_fp handler)
 {
-  struct imap_auth *p = malloc (sizeof (*p));
-
-  if (!p)
-    imap4d_bye (ERR_NO_MEM);
+  struct imap_auth *p = mu_alloc (sizeof (*p));
 
   p->name = name;
   p->handler = handler;
@@ -81,9 +78,10 @@ _auth_try (void *item, void *data)
 }
 
 void
-imap4d_auth_capability ()
+imap4d_auth_capability (struct imap4d_session *session)
 {
-  mu_list_foreach (imap_auth_list, _auth_capa, NULL);
+  if (session->tls_mode != tls_required)
+    mu_list_foreach (imap_auth_list, _auth_capa, NULL);
 }
 
 /*
@@ -93,18 +91,19 @@ imap4d_auth_capability ()
 */
 
 int
-imap4d_authenticate (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_authenticate (struct imap4d_session *session,
+		     struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   char *auth_type;
   struct imap4d_auth adata;
-  enum imap4d_auth_result res;
+  int res;
   
   if (imap4d_tokbuf_argc (tok) != 3)
     return io_completion_response (command, RESP_BAD, "Invalid arguments");
   
   auth_type = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
 
-  if (tls_required)
+  if (session->tls_mode == tls_required)
     return io_completion_response (command, RESP_NO,
 			           "Command disabled: Use STARTTLS first");
   
@@ -137,6 +136,10 @@ imap4d_authenticate (struct imap4d_command *command, imap4d_tokbuf_t tok)
     case imap4d_auth_fail:
       adata.response = RESP_NO;
       break;
+
+    default:
+      adata.response = RESP_NO;
+      mu_error ("%s", mu_strerror (res));
     }
   return io_completion_response (command, adata.response,
 				 "%s authentication failed", auth_type);
