@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2002, 2005-2007, 2009-2012, 2014-2016 Free
+   Copyright (C) 1999, 2001-2002, 2005-2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -37,119 +37,26 @@ dump_headers (mu_stream_t out, compose_env_t *env)
   mu_stream_destroy (&stream);
 }
 
-#define STATE_INIT 0
-#define STATE_READ 1
-#define STATE_BODY 2
-
 static int
-parse_headers (mu_stream_t input, compose_env_t *env)
+check_headers (mu_stream_t input, compose_env_t *env)
 {
-  int status;
-  mu_header_t header;
-  char *name = NULL;
-  char *value = NULL;
-  int state = STATE_INIT;
-  char *buf = NULL;
-  size_t size = 0, n;
-  int errcnt = 0, line = 0;
+  char *p;
   
-  if ((status = mu_header_create (&header, NULL, 0)) != 0)
-    {
-      mu_error (_("Cannot create header: %s"), mu_strerror (status));
-      return 1;
-    }
-
   mu_stream_seek (input, 0, MU_SEEK_SET, NULL);
-  while (state != STATE_BODY &&
-	 errcnt == 0 &&
-	 mu_stream_getline (input, &buf, &size, &n) == 0 && n > 0)
+  switch (parse_headers (input, env))
     {
-      mu_rtrim_class (buf, MU_CTYPE_SPACE);
-
-      line++;
-      switch (state)
-	{
-	case STATE_INIT:
-	  if (!buf[0] || mu_isspace (buf[0]))
-	    continue;
-	  else
-	    state = STATE_READ;
-	  /*FALLTHRU*/
-	  
-	case STATE_READ:
-	  if (buf[0] == 0)
-	    state = STATE_BODY;
-	  else if (mu_isspace (buf[0]))
-	    {
-	      /* A continuation line */
-	      if (name)
-		{
-		  char *p = NULL;
-		  mu_asprintf (&p, "%s\n%s", value, buf);
-		  free (value);
-		  value = p;
-		}
-	      else
-		{
-		  mu_error (_("%d: not a header line"), line);
-		  errcnt++;
-		}
-	    }
-	  else
-	    {
-	      char *p;
-	      
-	      if (name)
-		{
-		  mu_header_set_value (header, name, value[0] ? value : NULL, 0);
-		  free (name);
-		  free (value);
-		  name = value = NULL;
-		}
-	      p = strchr (buf, ':');
-	      if (p)
-		{
-		  *p++ = 0;
-		  while (*p && mu_isspace (*p))
-		    p++;
-		  value = mu_strdup (p);
-		  name = mu_strdup (buf);
-		}
-	      else
-		{
-		  mu_error (_("%d: not a header line"), line);
-		  errcnt++;
-		}
-	    }
-	  break;
-	}
+    case parse_headers_ok:
+      return 0;
+    case parse_headers_fatal:
+      return -1;
+    case parse_headers_error:
+      break;
     }
   
-  free (buf);
-  if (name)
-    {
-      mu_header_set_value (header, name, value, 0);
-      free (name);
-      free (value);
-    }     
-
-  if (errcnt)
-    {
-      char *p;
-      
-      mu_header_destroy (&header);
-      p = ml_readline (_("Edit again?"));
-      if (mu_true_answer_p (p) == 1)
-	return -1;
-      else
-	return 1;
-    }
-
-  mu_header_destroy (&env->header);
-  env->header = header;
-  return 0;
-}
-
+  p = ml_readline (_("Edit again?"));
+  return mu_true_answer_p (p);
+}  
+
 static void
 escape_continue (void)
 {
@@ -386,7 +293,7 @@ escape_run_editor (char *ed, int argc, char **argv, compose_env_t *env)
 	      return rc;
 	    }
 	}
-      while ((rc = parse_headers (tempstream, env)) < 0);
+      while (check_headers (tempstream, env));
     }
   else
     {
