@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -610,7 +610,7 @@ newfmt_init (struct xfer_format *fmt, const char *version,
   else
     {
       mu_opool_t pool;
-      rc = mu_opool_create (&pool, 0);
+      rc = mu_opool_create (&pool, MU_OPOOL_DEFAULT);
       if (rc)
 	{
 	  mu_diag_funcall (MU_DIAG_ERROR, "mu_opool_create",
